package com.dukascopy.indicators;

import com.dukascopy.api.IIndicators;
import com.dukascopy.api.indicators.IIndicator;
import com.dukascopy.api.indicators.IIndicatorContext;
import com.dukascopy.api.indicators.IndicatorInfo;
import com.dukascopy.api.indicators.IndicatorResult;
import com.dukascopy.api.indicators.InputParameterInfo;
import com.dukascopy.api.indicators.IntegerRangeDescription;
import com.dukascopy.api.indicators.OptInputParameterInfo;
import com.dukascopy.api.indicators.OutputParameterInfo;

import java.awt.Color;

/**
 * @author chriz aka Indiana Pips
 * Original idea by SkyStriker http://www.forexfactory.com/showthread.php?t=277168
 * Date: Jan 26, 2011
 */

public class MacdRiskie implements IIndicator {
    private IndicatorInfo indicatorInfo;
    private InputParameterInfo[] inputParameterInfos;
    private OptInputParameterInfo[] optInputParameterInfos;
    private OutputParameterInfo[] outputParameterInfos;
    private double[][][] inputs = new double[1][][];
    private double[][] outputs = new double[5][];
    private int fastPeriod = 12;
    private int slowPeriod = 26;
    private int signalPeriod = 9;

    public static final Color GREEN = new Color(0x00, 0x80, 0x00);
    public static final Color RED = new Color(0xc0, 0x00, 0x00);
    public static final Color LIGHT_GREEN = new Color(0x80, 0xFF, 0x80);
    public static final Color LIGHT_RED  = new Color(0xFF, 0x80, 0x80);
    public static final Color VERY_DARK_YELLOW = new Color(0x80, 0x80, 0x00);
    
    // fast emas        
    private IIndicator ema1;
    private IIndicator ema12;
    private IIndicator ema13;
    // slow emas    
    private IIndicator ema2;
    private IIndicator ema22;
    private IIndicator ema23;
    
    private IIndicator signalMa;

    public void onStart(IIndicatorContext context) {
        // fast emas
        ema1 = context.getIndicatorsProvider().getIndicator("EMA");
        ema12 = context.getIndicatorsProvider().getIndicator("EMA");
        ema13 = context.getIndicatorsProvider().getIndicator("EMA");
        // slow emas
        ema2 = context.getIndicatorsProvider().getIndicator("EMA");        
        ema22 = context.getIndicatorsProvider().getIndicator("EMA");
        ema23 = context.getIndicatorsProvider().getIndicator("EMA");
        // signal ma
        signalMa = context.getIndicatorsProvider().getIndicator("SMA");

        indicatorInfo = new IndicatorInfo("MacdRisk", "Macd Risk", "--nobody",
                false, false, true, 1, 3, 5);

        inputParameterInfos = new InputParameterInfo[] {new InputParameterInfo("Price", InputParameterInfo.Type.PRICE)};        
        
        //indicatorInfo.setRecalculateAll(true);

        optInputParameterInfos = new OptInputParameterInfo[] {
            new OptInputParameterInfo("Fast Period", OptInputParameterInfo.Type.OTHER,
                new IntegerRangeDescription(12, 2, 100, 1)),
           new OptInputParameterInfo("Slow Period", OptInputParameterInfo.Type.OTHER,
                new IntegerRangeDescription(26, 2, 100, 1)),
           new OptInputParameterInfo("Signal Period", OptInputParameterInfo.Type.OTHER,
                new IntegerRangeDescription(9, 1, 100, 1))
                };

        outputParameterInfos = new OutputParameterInfo[] {
                new OutputParameterInfo("Macd Up", OutputParameterInfo.Type.DOUBLE,OutputParameterInfo.DrawingStyle.HISTOGRAM)
                {{
                    setColor(GREEN);setShowValueOnChart(true);
                }},
                new OutputParameterInfo("Macd Up", OutputParameterInfo.Type.DOUBLE,OutputParameterInfo.DrawingStyle.HISTOGRAM)
                {{
                    setColor(LIGHT_GREEN);setShowValueOnChart(true);
                }},                
                new OutputParameterInfo("Macd Down", OutputParameterInfo.Type.DOUBLE,OutputParameterInfo.DrawingStyle.HISTOGRAM)
                {{
                    setColor(RED);setShowValueOnChart(true);
                }},
                new OutputParameterInfo("Macd Down", OutputParameterInfo.Type.DOUBLE,OutputParameterInfo.DrawingStyle.HISTOGRAM)
                {{
                    setColor(LIGHT_RED);setShowValueOnChart(true);
                }},
                new OutputParameterInfo("Signal Line", OutputParameterInfo.Type.DOUBLE,OutputParameterInfo.DrawingStyle.LINE)
                {{
                    setColor(VERY_DARK_YELLOW);setShowValueOnChart(true);
                }}                               
            };

    }

    public IndicatorResult calculate(int startIndex, int endIndex) {        
        if (startIndex - getLookback() < 0) {
            startIndex -= startIndex - getLookback();
        }
        if (startIndex > endIndex) {
            return new IndicatorResult(0, 0);
        }
        
        // fast emas
        ema1.setOptInputParameter(0, fastPeriod);
        ema12.setOptInputParameter(0, fastPeriod);
        ema13.setOptInputParameter(0, fastPeriod);
        // slow emas
        ema2.setOptInputParameter(0, slowPeriod);
        ema22.setOptInputParameter(0, slowPeriod);
        ema23.setOptInputParameter(0, slowPeriod);
        
        double[] emaA1 = new double[endIndex - startIndex + 1 + signalMa.getLookback()];
        double[] emaA12 = new double[endIndex - startIndex + 1 + signalMa.getLookback()];
        double[] emaA13 = new double[endIndex - startIndex + 1 + signalMa.getLookback()];
        
        double[] emaA2 = new double[endIndex - startIndex + 1 + signalMa.getLookback()];
        double[] emaA22 = new double[endIndex - startIndex + 1 + signalMa.getLookback()];
        double[] emaA23 = new double[endIndex - startIndex + 1 + signalMa.getLookback()];
        
        double[] macdOut = new double[endIndex  - startIndex + 1 + signalMa.getLookback()];
        double[] differenceOut = new double[endIndex  - startIndex + 1 + signalMa.getLookback()];
        
        double[] smaA3 = new double[endIndex - startIndex + 1];

        // ema on close
        ema1.setInputParameter(0, inputs[0][1]);
        ema2.setInputParameter(0, inputs[0][1]);
        ema1.setOutputParameter(0, emaA1);
        ema2.setOutputParameter(0, emaA2);
        // calculating
        ema1.calculate(startIndex - signalMa.getLookback(), endIndex);
        IndicatorResult ema2Result = ema2.calculate(startIndex - signalMa.getLookback(), endIndex);

        // ema on high
        ema12.setInputParameter(0, inputs[0][2]);
        ema22.setInputParameter(0, inputs[0][2]);
        ema12.setOutputParameter(0, emaA12);
        ema22.setOutputParameter(0, emaA22);
        // calculating
        ema12.calculate(startIndex - signalMa.getLookback(), endIndex);        
        ema22.calculate(startIndex - signalMa.getLookback(), endIndex);        

        // ema on low
        ema13.setInputParameter(0, inputs[0][3]);
        ema23.setInputParameter(0, inputs[0][3]);
        ema13.setOutputParameter(0, emaA13);
        ema23.setOutputParameter(0, emaA23);
        // calculating
        ema13.calculate(startIndex - signalMa.getLookback(), endIndex);        
        ema23.calculate(startIndex - signalMa.getLookback(), endIndex);        
             
                   
        // set signal ma period    
        signalMa.setOptInputParameter(0, signalPeriod);
        int z; 
        for (z = 0; z < ema2Result.getNumberOfElements(); z++){
            macdOut[z] = emaA1[z] - emaA2[z];
            double difference = 0;
            difference = ( (emaA1[z]-emaA13[z])/(emaA12[z]-emaA1[z]) )
            -
            ( (emaA2[z]-emaA23[z])/(emaA22[z]-emaA2[z]) ) ;            
            differenceOut[z] = difference ;
        }

        signalMa.setInputParameter(0, macdOut);
        signalMa.setOutputParameter(0, smaA3);
      
        IndicatorResult signalMAResult = signalMa.calculate(0, ema2Result.getNumberOfElements() - 1);
        
        double[] macdOutShortened = new double[signalMAResult.getNumberOfElements()];
        System.arraycopy(macdOut, signalMAResult.getFirstValueIndex(), macdOutShortened, 0, signalMAResult.getNumberOfElements());
        double[] differenceOutShortened = new double[signalMAResult.getNumberOfElements()];
        System.arraycopy(differenceOut, signalMAResult.getFirstValueIndex(), differenceOutShortened, 0, signalMAResult.getNumberOfElements());
                
        int i, k;
        for (i = 0, k = (signalMAResult.getNumberOfElements()); i < k; i++) {            
        
            double difference, macd ;
            macd = macdOutShortened[i];
            difference = differenceOutShortened[i];
             
            // set signal ma to output
            outputs[4][i] = smaA3[i] ;
            
            // uptrend
            if(difference>0)
            {
                if(macd>0)
                {
                    outputs[0][i] = macd ;
                    outputs[1][i] = Double.NaN ;
                    outputs[2][i] = Double.NaN ;
                    outputs[3][i] = Double.NaN ;   
                                    
                }else if(macd<=0)
                {
                    outputs[0][i] = Double.NaN ;
                    outputs[1][i] = Double.NaN ;
                    outputs[2][i] = Double.NaN ;
                    outputs[3][i] = macd;                       
                }                
            }else if(difference<0){ // downtrend
                if(macd>0)
                {
                    outputs[0][i] = Double.NaN  ;
                    outputs[1][i] = macd;
                    outputs[2][i] = Double.NaN ;
                    outputs[3][i] = Double.NaN ;   
                                    
                }else if(macd<=0)
                {
                    outputs[0][i] = Double.NaN ;
                    outputs[1][i] = Double.NaN ;
                    outputs[2][i] = macd ;
                    outputs[3][i] = Double.NaN ;                       
                }                                                       
            }                                                         
            
        }      
        return new IndicatorResult(ema2Result.getFirstValueIndex() + signalMAResult.getFirstValueIndex(), signalMAResult.getNumberOfElements());
    }

    public IndicatorInfo getIndicatorInfo() {
        return indicatorInfo;
    }

    public InputParameterInfo getInputParameterInfo(int index) {
        if (index <= inputParameterInfos.length) {
            return inputParameterInfos[index];
        }
        return null;
    }

    public int getLookback() {        
        return Math.max(ema1.getLookback(), ema2.getLookback()) + signalMa.getLookback();
    }

    public int getLookforward() {
        return 0;
    }

    public OptInputParameterInfo getOptInputParameterInfo(int index) {
        if (index <= optInputParameterInfos.length) {
            return optInputParameterInfos[index];
        }
        return null;
    }

    public OutputParameterInfo getOutputParameterInfo(int index) {
        if (index <= outputParameterInfos.length) {
            return outputParameterInfos[index];
        }
        return null;
    }

    public void setInputParameter(int index, Object array) {
        inputs[index] = (double[][]) array;
    }

    public void setOptInputParameter(int index, Object value) {
        if (index == 0) {
            fastPeriod = (Integer) value;
            ema1.setOptInputParameter(0, fastPeriod);
            ema12.setOptInputParameter(0, fastPeriod);
            ema13.setOptInputParameter(0, fastPeriod);
        } 
        if (index == 1) {
            slowPeriod = (Integer) value;
            ema2.setOptInputParameter(0, slowPeriod);
            ema22.setOptInputParameter(0, slowPeriod);
            ema23.setOptInputParameter(0, slowPeriod);
        }
        if (index == 2) {
            signalPeriod = (Integer) value;
            signalMa.setOptInputParameter(0, signalPeriod);
        }        
    }

    public void setOutputParameter(int index, Object array) {
        outputs[index] = (double[]) array;
    }
}